package com.vendor.burn.serial.workers;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.vendor.burn.model.PartitionItem;
import com.vendor.burn.model.define.IBurner;
import com.vendor.burn.model.devices.unknown.UnknownDevice;
import com.vendor.burn.model.event.EventType;
import com.vendor.burn.serial.workers.CallLibJTAGWorker;
import com.vendor.burn.serial.workers.PartitionBurnerWorker;
import com.vendor.burn.sparse.SparseFileAgent;
import com.vendor.burn.utilitily.FileUtils;
import com.vendor.platform.api.statistics.ToolStatistics;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.List;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.swt.widgets.Display;

public class MergeBurnerWorker extends PartitionBurnerWorker {
  private long lestLen = 0L;
  
  private boolean isDelete = false;
  
  public MergeBurnerWorker(IBurner burner) {
    super(burner);
    this.burner = burner;
  }
  
  public boolean mergeImage(IProgressMonitor uiMonitor, File file, List<PartitionItem> parts, boolean call) {
    ToolStatistics.INSTANCE.logUsage("BurnTool", "Merge Image", "ChipName: " + this.burner
        .getBurnContext().getDeviceSetting().getName());
    ToolStatistics.INSTANCE.logToolInUse("BurnTool");
    String versionInfo = getVersionInfo("Merge Image");
    CallLibJTAGWorker.printDebugInfo(versionInfo);
    String fillData = getFillData();
    monitor.getEvent().init(EventType.CREATE_IMAGE);
    if (uiMonitor.isCanceled()) {
      monitor.getEvent().setUserCanceled(true);
      return false;
    } 
    uiMonitor.beginTask("Creating Image", 100 * parts.size() + 3);
    RandomAccessFile imagefile = null;
    try {
      imagefile = new RandomAccessFile(file, "rw");
      int size = parts.size();
      long start = 0L;
      for (int i = 0; i < size; i++) {
        if (uiMonitor.isCanceled()) {
          monitor.getEvent().setUserCanceled(true);
          return false;
        } 
        PartitionItem part = parts.get(i);
        if (part.isChecked()) {
          long partitionLen = part.getLength();
          File partitionItemfile = part.getFile();
          File partfile = getPartfile(part, partitionItemfile);
          long curentStrat = part.getStart();
          long diffAddresses = 0L;
          if (curentStrat > start) {
            diffAddresses = curentStrat - start;
            imagefile.seek(start);
            fillDataToEmptyPart(fillData, imagefile, diffAddresses);
            start += diffAddresses;
          } 
          if (partfile != null && partfile.exists()) {
            imagefile.seek(start);
            fillDataToEmptyPart(fillData, imagefile, partitionLen);
            writeToFile(SubMonitor.convert(uiMonitor, "", 100), imagefile, start, partfile);
            start += partitionLen;
          } else {
            imagefile.seek(start);
            fillDataToEmptyPart(fillData, imagefile, partitionLen);
            start += partitionLen;
          } 
          if (this.isDelete)
            FileUtils.deleteFile(partfile); 
          uiMonitor.worked(100);
        } 
      } 
      infoln("The merged image generated successfully!");
      infoln(file.getCanonicalPath());
    } catch (IOException iOException) {
    
    } finally {
      if (imagefile != null)
        try {
          imagefile.close();
        } catch (IOException iOException) {} 
    } 
    return true;
  }
  
  private File getPartfile(PartitionItem part, File partitionItemfile) {
    File partfile = null;
    boolean isSparse = (SparseFileAgent.isSparse(part.getFileSystem()) && SparseFileAgent.isAndroidSparseFile(partitionItemfile));
    if (isSparse) {
      partfile = SparseFileAgent.unsparse(partitionItemfile);
      if (partfile != null) {
        try {
          infoln(partitionItemfile.getCanonicalPath() + " unsparse image successfully!");
        } catch (IOException iOException) {}
        this.isDelete = true;
      } 
    } else {
      partfile = partitionItemfile;
      this.isDelete = false;
    } 
    return partfile;
  }
  
  public boolean checkLastPartition(List<PartitionItem> parts) {
    long lastPartLen = getLastPartitionLength(parts);
    int size = parts.size();
    PartitionItem partition = parts.get(size - 1);
    if (partition.getLength() <= 0L) {
      if (lastPartLen == -1L)
        return false; 
      if (lastPartLen % 1024L != 0L) {
        Display.getDefault().syncExec((Runnable)new Object(this));
        return false;
      } 
      if (0L == lastPartLen) {
        Display.getDefault().syncExec((Runnable)new Object(this));
        return false;
      } 
      if (lastPartLen < 0L) {
        Display.getDefault().syncExec((Runnable)new Object(this));
        return false;
      } 
      partition.setLength(lastPartLen);
      if (getMonitor() != null)
        getMonitor().refreshUiPartitionTable(); 
    } 
    return true;
  }
  
  public long getLastPartitionLength(List<PartitionItem> parts) {
    int emmcType = ((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).getEmmcType();
    int size = parts.size();
    PartitionItem partition = parts.get(size - 1);
    long partitionLength = partition.getLength();
    long start = partition.getStart();
    if (partitionLength == -1L)
      Display.getDefault().syncExec((Runnable)new Object(this, size, parts, emmcType, start)); 
    return this.lestLen;
  }
}
